local tablet = AceLibrary("Tablet-2.0")
local L = AceLibrary("AceLocale-2.2"):new("ErrorMonsterFu")

L:RegisterTranslations("enUS", function() return {
	["Last flush"] = true,
	["Last message"] = true,
	["Messages"] = true,
	["Flushes"] = true,

	["This session"] = true,
	["Total"] = true,
} end)

local playerName = nil
local lastFlushed = nil
local lastMessage = nil
local sessionFlushes = nil
local sessionMessages = nil

ErrorMonsterFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceEvent-2.0")
ErrorMonsterFu.hasIcon = "Interface\\Icons\\INV_Food_ChristmasFruitCake_01"
ErrorMonsterFu.hasNoColor = true
ErrorMonsterFu:RegisterDB("ErrorMonsterFuDB", "ErrorMonsterFuCharDB")
ErrorMonsterFu:RegisterDefaults("char", {
	flushes = 0,
	messages = 0,
})
ErrorMonsterFu:RegisterDefaults("realm", {
	chars = {},
})

function ErrorMonsterFu:OnEnable()
	self:RegisterEvent("ErrorMonster_MessageFlushed")
	self:RegisterEvent("ErrorMonster_Message")

	playerName = UnitName("player")
	sessionMessages = 0
	sessionFlushes = 0

	self:UpdateDisplay()
end

function ErrorMonsterFu:IncreaseMessages()
	self.db.char.messages = self.db.char.messages + 1
	if not self.db.realm.chars[playerName] then
		self.db.realm.chars[playerName] = {}
		self.db.realm.chars[playerName].flushes = 0
		self.db.realm.chars[playerName].messages = 0
	end
	self.db.realm.chars[playerName].messages = self.db.realm.chars[playerName].messages + 1
	sessionMessages = sessionMessages + 1

	self:UpdateDisplay()
end

function ErrorMonsterFu:IncreaseFlushes()
	self.db.char.flushes = self.db.char.flushes + 1
	if not self.db.realm.chars[playerName] then
		self.db.realm.chars[playerName] = {}
		self.db.realm.chars[playerName].flushes = 0
		self.db.realm.chars[playerName].messages = 0
	end
	self.db.realm.chars[playerName].flushes = self.db.realm.chars[playerName].flushes + 1
	sessionFlushes = sessionFlushes + 1

	self:UpdateDisplay()
end

function ErrorMonsterFu:ErrorMonster_Message(message)
	lastMessage = message
	self:IncreaseMessages()
end

function ErrorMonsterFu:ErrorMonster_MessageFlushed(message)
	lastFlushed = message
	self:IncreaseFlushes()
end

function ErrorMonsterFu:OnTextUpdate()
	self:SetText(tostring(sessionMessages) .. "/" .. tostring(sessionFlushes))
end

function ErrorMonsterFu:OnTooltipUpdate()
	local totalFlushes = 0
	local totalMessages = 0

	local textCat = tablet:AddCategory(
		"columns", 2,
		"child_textR", 1,
		"child_textG", 1,
		"child_textB", 0,
		"child_text2R", 1,
		"child_text2G", 1,
		"child_text2B", 1,
		"showWithoutChildren", false
	)
	if lastFlushed then textCat:AddLine("text", L["Last flush"], "text2", lastFlushed) end
	if lastMessage then textCat:AddLine("text", L["Last message"], "text2", lastMessage) end

	local statCat = tablet:AddCategory(
		"columns", 2,
		"child_child_justify", "LEFT",
		"child_child_justify2", "RIGHT",
		"showWithoutChildren", false
	)

	local session = statCat:AddCategory("text", L["This session"])
	session:AddLine("text", L["Flushes"], "text2", sessionFlushes)
	session:AddLine("text", L["Messages"], "text2", sessionMessages)

	local character = statCat:AddCategory("text", playerName)
	character:AddLine("text", L["Flushes"], "text2", self.db.char.flushes)
	character:AddLine("text", L["Messages"], "text2", self.db.char.messages)

	totalMessages = self.db.char.messages
	totalFlushes = self.db.char.flushes

	for k, v in pairs(self.db.realm.chars) do
		if k ~= playerName then
			local cat = statCat:AddCategory("text", k)
			cat:AddLine("text", L["Flushes"], "text2", self.db.realm.chars[k].flushes)
			cat:AddLine("text", L["Messages"], "text2", self.db.realm.chars[k].messages)
			totalMessages = totalMessages + self.db.realm.chars[k].messages
			totalFlushes = totalFlushes + self.db.realm.chars[k].flushes
		end
	end

	local total = statCat:AddCategory("text", L["Total"])
	total:AddLine("text", L["Flushes"], "text2", totalFlushes)
	total:AddLine("text", L["Messages"], "text2", totalMessages)
end

